@@ -18,6 +18,38 @@
 NOTES:
 We keep track of a delete list while the history menu is up. We add/remove items from that 
 list until we get a selection done event, then we delete those items from the real history
+
+From http://standards.freedesktop.org/clipboards-spec/clipboards-latest.txt
+
+Application authors should follow the following guidelines to get
+correct behavior:
+
+ - selecting but with no explicit copy should only set PRIMARY,
+   never CLIPBOARD
+
+ - middle mouse button should paste PRIMARY, never CLIPBOARD
+
+ - explicit cut/copy commands (i.e. menu items, toolbar buttons)
+   should always set CLIPBOARD to the currently-selected data (i.e.
+   conceptually copy PRIMARY to CLIPBOARD)
+
+ - explicit cut/copy commands should always set both CLIPBOARD and
+   PRIMARY, even when copying doesn't involve a selection (e.g. a
+   "copy url" -option which explicitly copies an url without the
+   url being selected first)
+
+ - explicit paste commands should paste CLIPBOARD, not PRIMARY
+
+ - a selection becoming unselected should never unset PRIMARY
+
+ - possibly contradicting the ICCCM, clients don't need to support
+   SECONDARY, though if anyone can figure out what it's good
+   for they should feel free to use it for that
+
+ - cut buffers are evil; they only support ASCII, they don't 
+   work with many clients, and they require data to be 
+   copied to the X server. Therefore clients should avoid 
+   using cut buffers and use only selections.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -26,17 +58,39 @@ list until we get a selection done event, then we delete those items from the re
 
 #include "parcellite.h"
 
+/**ACT are actions, and MODE is the mode of the action  */
+/** #define ACT_STOP  0
+#define ACT_RUN   1 */
 
+/** defines the mode of the above actions. These are bit-wise */
+#define CMODE_PRI 1
+#define	CMODE_CLI 2
+#define CMODE_ALL 3 /**needs to be or of the above  */
+
+#define FIFCMD_STOP_PRI "stop_pri"
+#define FIFCMD_STOP_CLI "stop_cli"
+#define FIFCMD_STOP_ALL "stop_all"
+#define FIFCMD_RUN_PRI "run_pri"
+#define FIFCMD_RUN_CLI "run_cli"
+#define FIFCMD_RUN_ALL "run_all"
 
 GtkWidget *hmenu;
 /* Uncomment the next line to print a debug trace. */
-/*#define DEBUG   */
+/*#define DEBUG    */
 
 #ifdef DEBUG
 #  define TRACE(x) x
 #else
 #  define TRACE(x) do {} while (FALSE);
 #endif
+/**see parcellite.h for define  */
+#ifdef DEBUG_UPDATE
+static int debug_update=0; /**disable/enable DTRACE  */
+#  define DTRACE(x) if (debug_update) x
+#else
+#  define DTRACE(x) do {} while (FALSE);
+#endif
+
 static GtkClipboard* primary;
 static GtkClipboard* clipboard;
 struct p_fifo *fifo;
@@ -50,7 +104,10 @@ GMutex *hist_lock=NULL;
 static gboolean actions_lock = FALSE;
 static int show_icon=0;
 static int have_appindicator=0; /**if set, we have a running indicator-appmenu  */
-static gchar *appindicator_process="indicator-appmenu"; /**process name  */
+static gchar *appindicator_process="indicator-messages-service"; /**process name  */
+
+static int cmd_mode=CMODE_ALL; /**both clipboards  */
+/** static int cmd_state=ACT_RUN; running  */
 /**defines for moving between clipboard histories  */
 #define HIST_MOVE_TO_CANCEL     0
 #define HIST_MOVE_TO_OK         1
@@ -97,7 +154,7 @@ gchar *process_new_item(GtkClipboard *clip,gchar *ntext)
 	if(NULL == ntext)
 		return NULL;
 	
-				
+	
 /**we now check our options...  */		
 	/*printf("opt\n"); fflush(NULL); */
 	if (get_pref_int32("hyperlinks_only")){
@@ -152,18 +209,23 @@ done:
 gchar *_update_clipboard (GtkClipboard *clip, gchar *n, gchar **old, int set)
 {
 	
+	/*return NULL; */
 	if(NULL != n)	{
-	/**  	if(clip==primary)
+#ifdef DEBUG_UPDATE
+	 	if(clip==primary)
 			g_printf("set PRI to %s\n",n);
 		else
-			g_printf("set CLI to %s\n",n);*/
+			g_printf("set CLI to %s\n",n);
+#endif
 		if( set)
 			gtk_clipboard_set_text(clip, n, -1);
-		if(NULL != *old)
-		  g_free(*old);
-		*old=g_strdup(n);
-		return *old;
-	}else{
+		if(NULL != old ){
+			if( NULL != *old)
+		  	g_free(*old);
+			*old=g_strdup(n);
+			return *old;
+		}	
+	}else if( NULL != old){
 		if(NULL != *old)
 		  g_free(*old);
 		*old=NULL;
@@ -172,23 +234,44 @@ gchar *_update_clipboard (GtkClipboard *clip, gchar *n, gchar **old, int set)
 	return NULL;
 }
 
-
 /***************************************************************************/
 /** .
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
-gboolean is_clipboard_empty(GtkClipboard *clip)
+gboolean content_exists(GtkClipboard *clip)
 {
-  int count;
+	gint count;
+	GdkAtom *targets;
+	gboolean contents = gtk_clipboard_wait_for_targets(clip, &targets, &count);
+	g_free(targets);
+	return contents;
+}
+
+
+/***************************************************************************/
+/** This DOES NOT WORK!! WTH??.
+Fix is a bit hacky.
+\n\b Arguments:
+\n\b Returns:	 clipboard contents, to be freed with g_free, or NULL if empty.
+****************************************************************************/
+gchar *is_clipboard_empty(GtkClipboard *clip)
+{
+  /** int count;
   GdkAtom *targets;
+	if(clipboard == clip) g_printf("-%s-",gtk_clipboard_wait_for_text(clip));
   gboolean contents = gtk_clipboard_wait_for_targets(clip, &targets, &count);
+	if(clipboard == clip) g_printf("-%s-2nd-",gtk_clipboard_wait_for_text(clip));
   g_free(targets);
-  return !contents;
+	if(TRUE == contents || count >0)
+		return 0;*/
+	if(TRUE == gtk_clipboard_wait_is_text_available(clip))
+		return(gtk_clipboard_wait_for_text(clip));
+	return NULL;
 }
 
 /***************************************************************************/
-/** .
+/** Update one clipboard at a time.
 \n\b Arguments:
 \n\b Returns:
 ****************************************************************************/
@@ -213,14 +296,15 @@ gchar *update_clipboard(GtkClipboard *clip,gchar *intext,  gint mode)
 		existing=&ctext;
 	}
 	
-	/** if(H_MODE_CHECK!=mode )
-	g_printf("HC%d-%c: in %s,ex %s\n",mode,clip==primary?'p':'c',intext,*existing);*/
+	/** if(H_MODE_CHECK!=mode )	 */
+   /*g_printf("HC%d-%c: in %s,ex %s\n",mode,clip==primary?'p':'c',intext,*existing); */
 	if( H_MODE_INIT == mode){
 		if(NULL != *existing)
 			g_free(*existing);
 		*existing=NULL;
 		if(NULL != intext)
-		gtk_clipboard_set_text(clip, intext, -1);
+			_update_clipboard(clip,intext,NULL,1);
+/*		gtk_clipboard_set_text(clip, intext, -1); */
 		return NULL;
 	}
 	/**check that our clipboards are valid and user wants to use them  */
@@ -229,37 +313,64 @@ gchar *update_clipboard(GtkClipboard *clip,gchar *intext,  gint mode)
 		(clip == clipboard && !get_pref_int32("use_copy")))
 			return NULL;
 	
-	if(H_MODE_CHECK==mode &&clip == primary){/*fix auto-deselect of text in applications like DevHelp and LyX*/
+	if( H_MODE_CHECK==mode &&clip == primary){/*fix auto-deselect of text in applications like DevHelp and LyX*/
    	gdk_window_get_pointer(NULL, NULL, NULL, &button_state);
 		if ( button_state & (GDK_BUTTON1_MASK|GDK_SHIFT_MASK) ) /**button down, done.  */
 			goto done;
 	}
+	if(0 && NULL != intext){ /**we run this in process_new_item  */
+		validate_utf8_text(intext,strlen(intext));
+	}	
 	/*g_printf("BS=0x%02X ",button_state); */
 	if( H_MODE_IGNORE == mode){	/**ignore processing and just put it on the clip.  */
-/*		g_printf("%sJustSet '%s'\n",clipname,intext); */
-		gtk_clipboard_set_text(clip, intext, -1);
+		DTRACE(g_printf("%sJustSet '%s'\n",clip==clipboard?"CLI":"PRI",intext)); 
+		_update_clipboard(clip,intext,NULL,1);
+		/*gtk_clipboard_set_text(clip, intext, -1); */
 		return intext;
 	}
+	if(H_MODE_LIST == mode && 0 != p_strcmp(intext,*existing)){ /**just set clipboard contents. Already in list  */
+		DTRACE(g_printf("%sInList '%s' ex '%s'\n",clip==clipboard?"CLI":"PRI",intext,*existing)); 
+		last=_update_clipboard(clip,intext,existing,1);
+		if(NULL != last){/**maintain persistence, if set  */
+			append_item(last,get_pref_int32("current_on_top")?HIST_DEL|HIST_CHECKDUP|HIST_KEEP_FLAGS:0);
+		}
+		goto done;
+	}
 	/**check for lost contents and restore if lost */
-	/* Only recover lost contents if there isn't any other type of content in the clipboard */
-	if(is_clipboard_empty(clip) && NULL != *existing ) {
-/*		g_printf("%sclp empty, BS=0x%02X set to '%s'\n",clipname,button_state,*existing); */
-    gtk_clipboard_set_text(clip, *existing, -1);
-		last=*existing;
+	changed=is_clipboard_empty(clip);
+	if(NULL != changed){
+		validate_utf8_text(changed,strlen(changed));
+	}
+	
+	if(NULL != *existing && NULL == changed && 1 == get_pref_int32("restore_empty")) {
+		DTRACE(g_printf("%sclp empty, ",clip==clipboard?"CLI":"PRI"));
+		/* Only recover lost contents if there isn't any other type of content in the clipboard */
+		if (!content_exists(clip)) {
+			DTRACE(g_printf("set to '%s'\n",*existing));  
+			_update_clipboard(clip, *existing,NULL,1);
+    	/*gtk_clipboard_set_text(clip, *existing, -1); */
+			last=*existing;
+		}	else
+			DTRACE(g_printf("Left Null\n"));  
+		return *existing;
   }
+	if(NULL == changed)
+		return NULL;																	
 	/**check for changed clipboard content - in all modes */
-	changed=gtk_clipboard_wait_for_text(clip);
+	/*changed=gtk_clipboard_wait_for_text(clip); */
 	if(0 == p_strcmp(*existing, changed) ){
 		g_free(changed);                    /**no change, do nothing  */
 		changed=NULL;
 	}	else {
-/*		g_printf("%sclp changed: ex '%s' is '%s'\n",clipname,*existing,changed); */
-		if(NULL != (processed=process_new_item(clip,changed)) ){
-			if(0 == p_strcmp(processed,changed)) set=0;
-			else set=1;
-				 
+		DTRACE(g_printf("%sclp changed: ex '%s' is '%s' - ",clip==clipboard?"CLI":"PRI",*existing,changed)); 
+		if(NULL != (processed=process_new_item(clip,changed)) ){ 
+			/**only check processed/changed. No need to update this clip, since the text is already there.  */
+			 if(0 == p_strcmp(processed,changed)) set=0;
+			else set=1; 
+			DTRACE(g_printf("set=%d. p='%s'\n",set,processed));
+			/*set=1; */ /** Always set the text.*/
 			last=_update_clipboard(clip,processed,existing,set);
-		}else {/**restore clipboard  */
+		}else {/**restore clipboard - new item is binary/garbage/empty */
 			gchar *d;
 			
 			if(NULL ==*existing && NULL != history_list){
@@ -269,7 +380,7 @@ gchar *update_clipboard(GtkClipboard *clip,gchar *intext,  gint mode)
 			}else 
 				d=*existing;
 			if(NULL != d){
-/*				g_printf("%srestore clp '%s'\n",clipname,d); */
+				DTRACE(g_printf("\n%srestore clp '%s', ex='%s'\n",clip==clipboard?"CLI":"PRI",d,*existing)); 
 				last=_update_clipboard(clip,d,existing,1);
 			}
 				
@@ -287,14 +398,7 @@ gchar *update_clipboard(GtkClipboard *clip,gchar *intext,  gint mode)
 	
 	*/		
 	
-	if(H_MODE_LIST == mode && 0 != p_strcmp(intext,*existing)){ /**just set clipboard contents. Already in list  */
-/*		g_printf("%sInList '%s' ex '%s'\n",clipname,intext,*existing); */
-		last=_update_clipboard(clip,intext,existing,1);
-		if(NULL != last){/**maintain persistence, if set  */
-			append_item(last,get_pref_int32("current_on_top")?HIST_DEL|HIST_CHECKDUP|HIST_KEEP_FLAGS:0);
-		}
-		goto done;
-	}else if(H_MODE_NEW==mode){
+	if(H_MODE_NEW==mode){
 		if(NULL != (processed=process_new_item(clip,intext)) ){
 /*			g_printf("%sNEW '%s'\n",clipname,processed); */
 			if(0 == p_strcmp(processed,*existing))set=0;
@@ -321,6 +425,42 @@ void update_clipboards(gchar *intext, gint mode)
 	update_clipboard(primary, intext, mode);
 	update_clipboard(clipboard, intext, mode);
 }
+
+/***************************************************************************/
+/** Run a command. For now, just start and stop
+\n\b Arguments:
+\n\b Returns:
+****************************************************************************/
+void do_command(gchar *buf, gint len)
+{
+  g_printf("Got '%s' cmd\n",buf);
+	if(!p_strcmp(buf,FIFCMD_RUN_ALL)) {
+		cmd_mode|=CMODE_ALL;
+		return;
+	}	
+	if(!p_strcmp(buf,FIFCMD_RUN_CLI)) {
+		cmd_mode|=CMODE_CLI;
+		return;
+	}	
+	if(!p_strcmp(buf,FIFCMD_RUN_PRI)) {
+		cmd_mode|=CMODE_PRI;
+		return;
+	}	
+	if(!p_strcmp(buf,FIFCMD_STOP_ALL)) {
+		cmd_mode&=~(CMODE_ALL);
+		return;
+	}
+		if(!p_strcmp(buf,FIFCMD_STOP_CLI)) {
+		cmd_mode&=~(CMODE_CLI);
+		return;
+	}
+		if(!p_strcmp(buf,FIFCMD_STOP_PRI)) {
+		cmd_mode&=~(CMODE_PRI);
+		return;
+	}
+	
+}
+
 /***************************************************************************/
 /** Checks the clipboards and fifos for changes.
 \n\b Arguments:
@@ -330,8 +470,19 @@ void check_clipboards(gint mode)
 {
 	gchar *ptext, *ctext, *last;
 	int n=0;
+	
 	/*g_printf("check_clipboards\n"); */
 	/*g_mutex_lock(clip_lock); */
+	if(fifo->clen){/**we have a command to execute  */
+			/*fifo->which should be ID_CMD: */
+		if(fifo->dbg) g_printf("Running CMD '%s'\n",fifo->cbuf);
+		do_command(fifo->cbuf, fifo->clen);
+		if(fifo->dbg) g_printf("mode is 0x%X\n",cmd_mode);
+		fifo->clen=0;
+		return;
+	}	
+	if(!(CMODE_ALL & cmd_mode))
+		return;
 	if(fifo->rlen >0){
 		switch(fifo->which){
 			case ID_PRIMARY:
@@ -348,6 +499,7 @@ void check_clipboards(gint mode)
 				n=2;
 				fifo->rlen=0;
 				break;
+			
 			default:
 				fifo->rlen=validate_utf8_text(fifo->buf, fifo->rlen);
 				g_printf("CLIP not set, discarding '%s'\n",fifo->buf);
@@ -386,9 +538,12 @@ done:
 ****************************************************************************/
 gboolean check_for_appindictor( gpointer data)
 { 
+	int mode=PROC_MODE_STRSTR;
 	if(NULL != appindicator_process && !have_appindicator ){
-		g_printf("Looking for '%s'\n",appindicator_process);
-		if(proc_find(appindicator_process,PROC_MODE_STRSTR,NULL) >0){
+		/*g_printf("Looking for '%s'\n",appindicator_process); */
+		if(get_pref_int32("multi_user"))
+			mode|=PROC_MODE_USER_QUALIFY;
+		if(proc_find(appindicator_process,mode,NULL) >0){
 			have_appindicator=1;
 			if(NULL == indicator && show_icon)
 				create_app_indicator();	
@@ -405,6 +560,8 @@ gboolean check_for_appindictor( gpointer data)
 ****************************************************************************/
 gboolean check_clipboards_tic(gpointer data)
 {
+	/** gchar *txt=gtk_clipboard_wait_for_text(clipboard);
+	g_printf("%s\n",txt);*/
 	check_clipboards(H_MODE_CHECK);
 #ifdef HAVE_APPINDICATOR
 	if(have_appindicator){
@@ -459,9 +616,9 @@ static void action_selected(GtkButton *button, gpointer user_data)
   }
   /* Insert clipboard into command (user_data), and prepare it for execution */
   gchar* clipboard_text = gtk_clipboard_wait_for_text(clipboard);
-	/*g_print("Got cmd '%s', text '%s'->",(gchar *)user_data,clipboard_text);fflush(NULL);  */
+	g_print("Got cmd '%s', text '%s'->",(gchar *)user_data,clipboard_text);fflush(NULL);  
 	gchar* command=g_strdup_printf((gchar *)user_data,clipboard_text);
-	/*g_print(" '%s'\n",command);fflush(NULL);  */
+	g_print(" '%s'\n",command);fflush(NULL);  
   g_free(clipboard_text);
   g_free(user_data);
   gchar* shell_command = g_shell_quote(command);
@@ -473,7 +630,7 @@ static void action_selected(GtkButton *button, gpointer user_data)
   GPid pid;
   gchar **argv;
   g_shell_parse_argv(cmd, NULL, &argv, NULL);
-	/*g_print("cmd '%s' argv '%s' '%s' '%s'\n",cmd,argv[1],argv[2],argv[3]);  */
+	g_print("cmd '%s' argv '%s' '%s' '%s'\n",cmd,argv[1],argv[2],argv[3]);  
   g_free(cmd);
   g_spawn_async(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
   g_child_watch_add(pid, (GChildWatchFunc)action_exit, NULL);
@@ -730,6 +887,7 @@ void  history_item_right_click (struct history_info *h, GdkEventKey *e, gint ind
 /* Called when Clear is selected from history menu */
 static void clear_selected(GtkMenuItem *menu_item, gpointer user_data)
 {
+	int clear=1;
   /* Check for confirm clear option */
   if (get_pref_int32("confirm_clear"))  {
     GtkWidget* confirm_dialog = gtk_message_dialog_new(NULL,
@@ -738,23 +896,21 @@ static void clear_selected(GtkMenuItem *menu_item, gpointer user_data)
                                                        GTK_BUTTONS_OK_CANCEL,
                                                        _("Clear the history?"));
     
-    if (gtk_dialog_run((GtkDialog*)confirm_dialog) == GTK_RESPONSE_OK)    {
-      /* Clear history and free history-related variables */
-			clear_history();
-      
-		  update_clipboard(primary, "", H_MODE_INIT);
-      update_clipboard(clipboard, "", H_MODE_INIT);
-    }
-    gtk_widget_destroy(confirm_dialog);
-  }
-  else  {
-    /* Clear history and free history-related variables */
-    g_list_free(history_list);
-    history_list = NULL;
-    save_history();
-    update_clipboard(primary, "", H_MODE_INIT);
+    if (gtk_dialog_run((GtkDialog*)confirm_dialog) != GTK_RESPONSE_OK)    {
+			clear=0;
+		}
+		gtk_widget_destroy(confirm_dialog);
+	}	
+	if(clear){
+		struct history_info *h=(struct history_info *)user_data;
+		/* Clear history and free history-related variables */
+		remove_deleted_items(h); /**fix bug 92, Shift/ctrl right-click followed by clear segfaults/double free.  */	
+		clear_history();
+		/*g_printf("Clear hist done, h=%p, h->delete_list=%p\n",h, h->delete_list); */
+		update_clipboard(primary, "", H_MODE_INIT);
     update_clipboard(clipboard, "", H_MODE_INIT);
-  }
+	}
+      
 }
 
 /* Called when About is selected from right-click menu */
@@ -784,7 +940,7 @@ static void show_about_dialog(GtkMenuItem *menu_item, gpointer user_data)
                         gtk_widget_render_icon(about_dialog, GTK_STOCK_ABOUT, GTK_ICON_SIZE_MENU, NULL));
     
     gtk_about_dialog_set_name((GtkAboutDialog*)about_dialog, "Parcellite");
-    #ifdef HAVE_CONFIG_H
+    #ifdef HAVE_CONFIG_H	/**VER=555; sed "s#\(.*\)svn.*\".*#\1svn$VER\"#" config.h  */
     gtk_about_dialog_set_version((GtkAboutDialog*)about_dialog, VERSION);
     #endif
     gtk_about_dialog_set_comments((GtkAboutDialog*)about_dialog,
@@ -970,22 +1126,8 @@ static gboolean show_actions_menu(gpointer data)
 static gboolean selection_done(GtkMenuShell *menushell, gpointer user_data) 
 {
 	struct history_info *h=(struct history_info *)user_data;
-	/*g_print("Got selection_done shell %p data %p\n",menushell,user_data);  */
 	if(NULL != h && NULL != h->delete_list){/**have a list of items to delete.  */
-		GList *i;
-		/*g_print("Deleting items\n"); */
-		for (i=h->delete_list; NULL != i; i=i->next){
-			struct s_item_info *it=(struct s_item_info *)i->data;
-			/*printf("Free %p.. '%s' ",it->element->data,(char *)(it->((struct history_item *(element->data))->text))); */
-			g_free(it->element->data);
-			it->element->data=NULL;
-			history_list = g_list_delete_link(history_list, it->element);
-			/** printf("Free %p\n",it);
-			fflush(NULL);*/
-			g_free(it);
-		}
-		if (get_pref_int32("save_history"))
-		  save_history();
+		remove_deleted_items(h);
 		goto done;
 	}	
 	/*g_print("selection_active=%d\n",selection_active); */
@@ -996,7 +1138,9 @@ static gboolean selection_done(GtkMenuShell *menushell, gpointer user_data)
 	}
 		  
 done:
-	gtk_widget_destroy((GtkWidget *)menushell);
+	/*gtk_widget_destroy((GtkWidget *)menushell); - fixes annoying GTK_IS_WIDGET/GTK_IS_WINDOW
+	  warnings from GTK when history dialog is destroyed. */
+	return FALSE;
 }
 
 /***************************************************************************/
@@ -1161,7 +1305,7 @@ static gboolean key_release_cb (GtkWidget *w,GdkEventKey *e, gpointer user)
 			
 		else if(e->keyval == 'c'){
 			TRACE(g_print("Alt-C\n"));
-			clear_selected(NULL, NULL); 
+			clear_selected(NULL, (gpointer)h); 
 		}	else{
 			TRACE(g_print("Ignoring Alt-%c (0x%02x) state 0x%x",e->keyval,e->keyval,e->state));
 		}
@@ -1266,39 +1410,56 @@ foundit:
 ****************************************************************************/
 void set_clipboard_text(struct history_info *h, GList *element)
 {
+	int auto_whatever=0;
+	gchar *action=NULL;
+	gchar *txt=NULL;
+	gchar *cmd=NULL;
 	/*g_mutex_lock(clip_lock); */
 	if(NULL == find_h_item(h->delete_list,NULL,element)){	/**not in our delete list  */
 		/**make a copy of txt, because it gets freed and re-allocated.  */
-		gchar *txt=p_strdup(((struct history_item *)(element->data))->text);
-		/*g_printf("set_clip_text %s\n",txt);  */
+		txt=p_strdup(((struct history_item *)(element->data))->text);
+		DTRACE(("set_clip_text %s\n",txt));  
 		if(get_pref_int32("use_copy") )
 			update_clipboard(clipboard, txt, H_MODE_LIST);
 		if(get_pref_int32("use_primary"))
 	  	update_clipboard(primary, txt, H_MODE_LIST);	
-		g_free(txt);
+		
+		auto_whatever=1;
 	}
   g_signal_emit_by_name ((gpointer)h->menu,"selection-done");
+	if(0 == auto_whatever)
+		return;
 	/*g_printf("set_clip_text done\n");  */
 	/*g_mutex_unlock(clip_lock); */
 	
 	if (get_pref_int32("automatic_paste")) { /** mousedown 2 */
-		gchar *action=NULL;
 		if(get_pref_int32("auto_mouse"))
-			action="mousedown 2 && xdotool mouseup 2'";
+			action=g_strdup("mousedown 2 && xdotool mouseup 2'");
 		else if(get_pref_int32("auto_key"))
-			action="key ctrl+v'";
-		if(NULL == action)
-			return;
-		/**from clipit 1.4.1 */
-    gchar* cmd = g_strconcat("/bin/sh -c 'xdotool ", action, NULL);
-    GPid pid;
-    gchar **argv;
-    g_shell_parse_argv(cmd, NULL, &argv, NULL);
-    g_free(cmd);
-    g_spawn_async(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
-    g_child_watch_add(pid, (GChildWatchFunc)action_exit, NULL);
-    g_strfreev(argv);
-  }/**end from clipit 1.4.1 */
+			action=g_strdup("key ctrl+v'");
+	}
+	
+	if( get_pref_int32("key_input")) 
+		action=g_strconcat("type \"",txt,"\"'",NULL);
+		
+	if(NULL == action)
+			goto done;
+	/**from clipit 1.4.1 */
+  cmd = g_strconcat("/bin/sh -c 'xdotool ", action, NULL);
+	g_printf("xdotool:'%s'\ntext:'%s'\n",cmd,txt);
+  GPid pid;
+  gchar **argv;
+  g_shell_parse_argv(cmd, NULL, &argv, NULL);
+  g_free(cmd);
+  g_spawn_async(NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &pid, NULL);
+  g_child_watch_add(pid, (GChildWatchFunc)action_exit, NULL);
+  g_strfreev(argv);
+  /**end from clipit 1.4.1 */
+done:
+	if(NULL != txt)
+		g_free(txt);
+	if(NULL != action)
+		g_free(action);
 }
 
 
@@ -1400,6 +1561,39 @@ void write_history_menu_items(GList *list, GtkWidget *menu)
 	for (element = list; element != NULL; element = element->next) 
 			gtk_menu_shell_append((GtkMenuShell*)menu,element->data);	
 }
+
+/***************************************************************************/
+/** Replace non-printing characters with ??.
+0x09->E28692 - \2192
+0x0a->E2818B - \204b
+space-E290A3 - \u2423 
+\n\b Arguments:
+\n\b Returns:
+****************************************************************************/
+GString* convert_string(GString* s)
+{
+	gchar arrow[4]={0xe2,0x86,0x92,0x00};/**0xe28692 (UTF-8 right-arrow) \\  */
+	gchar pharagraph[4]={0xe2,0x81,0x8b,0x00}; /**utf-8 pharagraph symbol \2192 */
+	gchar square_u[4]={0xe2,0x90,0xA3,0};	 /**square-u \\2423  */
+	gchar *p, *r;
+	gint idx;
+	
+	for (p=s->str; p!= NULL; p=g_utf8_find_next_char(p,s->str+s->len)){
+		switch(*p){
+			case 0x09:r=arrow; break;
+			case 0x0a:r=pharagraph; break;
+			case 0x20:r=square_u; break;
+		  default:r=NULL; break; 
+		}
+		if(NULL !=r ) {/**replace. */
+			gint32 pos=(p-s->str)+1;
+			*p=r[0];
+			s=g_string_insert(s,pos,&r[1]);
+			p=s->str+pos+2;
+		}
+	}
+	return s;
+}
 /***************************************************************************/
 /**  Called when status icon is left-clicked or action key hit.
 \n\b Arguments:
@@ -1449,12 +1643,15 @@ static gboolean show_history_menu(gpointer data)
   /* Items */
   if ((history_list != NULL) && (history_list->data != NULL)) {
     /* Declare some variables */
-		
+		gint32 item_length= get_pref_int32("item_length");
+		gint32 ellipsize = get_pref_int32("ellipsize");
+		gint32 persistent_history=get_pref_int32("persistent_history");
+		gint32 nonprint_disp=get_pref_int32("nonprint_disp");
     gint element_number = 0;
     gchar* primary_temp = gtk_clipboard_wait_for_text(primary);
     gchar* clipboard_temp = gtk_clipboard_wait_for_text(clipboard);
     /* Reverse history if enabled */
-    if (get_pref_int32("reverse_history")) {
+    if (0 && get_pref_int32("reverse_history")) {
       /*history_list = g_list_reverse(history_list); */
       element_number = g_list_length(history_list) - 1;
     }
@@ -1467,19 +1664,21 @@ static gboolean show_history_menu(gpointer data)
 			else if( !(HIST_DISPLAY_NORMAL&h.histno) && !(c->flags & CLIP_TYPE_PERSISTENT))
 				goto next_loop;
       GString* string = g_string_new(hist_text);
+			if(nonprint_disp)
+				string=convert_string(string);
 		  glong len=g_utf8_strlen(string->str, string->len);
       /* Ellipsize text */
-      if (len > get_pref_int32("item_length")) {
-        switch (get_pref_int32("ellipsize")) {
+      if (len > item_length) {
+        switch (ellipsize) {
           case PANGO_ELLIPSIZE_START:
-          	string = g_string_erase(string, 0, g_utf8_offset_to_pointer(string->str, len - get_pref_int32("item_length")) - string->str);
+          	string = g_string_erase(string, 0, g_utf8_offset_to_pointer(string->str, len - item_length) - string->str);
             /*string = g_string_erase(string, 0, string->len-(get_pref_int32("item_length"))); */
             string = g_string_prepend(string, "...");
             break;
           case PANGO_ELLIPSIZE_MIDDLE:
           	{
-						gchar* p1 = g_utf8_offset_to_pointer(string->str, get_pref_int32("item_length") / 2);
-            gchar* p2 = g_utf8_offset_to_pointer(string->str, len - get_pref_int32("item_length") / 2);
+						gchar* p1 = g_utf8_offset_to_pointer(string->str, item_length / 2);
+            gchar* p2 = g_utf8_offset_to_pointer(string->str, len - item_length / 2);
             string = g_string_erase(string, p1 - string->str, p2 - p1);
             string = g_string_insert(string, p1 - string->str, "...");
             /** string = g_string_erase(string, (get_pref_int32("item_length")/2), string->len-(get_pref_int32("item_length")));
@@ -1487,7 +1686,7 @@ static gboolean show_history_menu(gpointer data)
 						}
             break;
           case PANGO_ELLIPSIZE_END:
-          	string = g_string_truncate(string, g_utf8_offset_to_pointer(string->str, get_pref_int32("item_length")) - string->str);
+          	string = g_string_truncate(string, g_utf8_offset_to_pointer(string->str, item_length) - string->str);
             /*string = g_string_truncate(string, get_pref_int32("item_length")); */
             string = g_string_append(string, "...");
             break;
@@ -1531,6 +1730,7 @@ static gboolean show_history_menu(gpointer data)
       if ((clipboard_temp) && (p_strcmp(hist_text, clipboard_temp) == 0))
       {
         gchar* bold_text = g_markup_printf_escaped("<b>%s</b>", string->str);
+			  if( NULL == bold_text) g_printf("NulBMKUp:'%s'\n",string->str);
         gtk_label_set_markup((GtkLabel*)item_label, bold_text);
         g_free(bold_text);
         h.clip_item=menu_item;
@@ -1539,12 +1739,13 @@ static gboolean show_history_menu(gpointer data)
       else if ((primary_temp) && (p_strcmp(hist_text, primary_temp) == 0))
       {
         gchar* italic_text = g_markup_printf_escaped("<i>%s</i>", string->str);
+			  if( NULL == italic_text) g_printf("NulIMKUp:'%s'\n",string->str);
         gtk_label_set_markup((GtkLabel*)item_label, italic_text);
         g_free(italic_text);
         h.clip_item=menu_item;
 			  h.element_text=hist_text;
       }
-			if(get_pref_int32("persistent_history") && c->flags &CLIP_TYPE_PERSISTENT){
+			if(persistent_history && c->flags &CLIP_TYPE_PERSISTENT){
 				persistent = g_list_prepend(persistent, menu_item);
 				/*g_printf("persistent %s\n",c->text); */
 			}	else{
@@ -1557,9 +1758,9 @@ static gboolean show_history_menu(gpointer data)
       /* Prepare for next item */
       g_string_free(string, TRUE);
 next_loop:
-      if (get_pref_int32("reverse_history"))
+      /** if (get_pref_int32("reverse_history"))
         element_number--;
-      else
+      else*/
         element_number++;
     }	/**end of for loop for each history item  */
     /* Cleanup */
@@ -1614,7 +1815,7 @@ next_loop:
 		/* Clear */
 	  menu_image = gtk_image_new_from_stock(GTK_STOCK_CLEAR, GTK_ICON_SIZE_MENU);
 	  gtk_image_menu_item_set_image((GtkImageMenuItem*)menu_item, menu_image);
-		g_signal_connect((GObject*)menu_item, "activate", (GCallback)clear_selected, NULL);
+		g_signal_connect((GObject*)menu_item, "activate", (GCallback)clear_selected, (gpointer)&h);
 	  gtk_menu_shell_append((GtkMenuShell*)menu, menu_item);
   }
 	g_list_free(lhist);
@@ -1623,7 +1824,10 @@ next_loop:
   gtk_widget_show_all(menu);
   gtk_menu_popup((GtkMenu*)menu, NULL, NULL, get_pref_int32("history_pos")?postition_history:NULL, NULL, 1, gtk_get_current_event_time());
 	/**set last entry at first -fixes bug 2974614 */
-	gtk_menu_shell_select_first((GtkMenuShell*)menu, TRUE);
+	if(get_pref_int32("reverse_history") && NULL != h.clip_item)
+		gtk_menu_shell_select_item((GtkMenuShell*)menu,h.clip_item);
+	else	
+		gtk_menu_shell_select_first((GtkMenuShell*)menu, TRUE);
   /* Return FALSE so the g_timeout_add() function is called only once */
   return FALSE;
 }
@@ -1807,20 +2011,22 @@ static void parcellite_init()
 	clip_lock= g_mutex_new();
 	hist_lock= g_mutex_new();
 	g_mutex_unlock(clip_lock);
-  /* Read preferences */
-  read_preferences();
+
   show_icon=!get_pref_int32("no_icon");
   /* Read history */
   if (get_pref_int32("save_history")){
+		gchar *x;
 		/*g_printf("Calling read_hist\n"); */
 		read_history();
 		if(NULL != history_list){
 			struct history_item *c;
 			c=(struct history_item *)(history_list->data);	
-			if(is_clipboard_empty(primary))
+			if(NULL == (x=is_clipboard_empty(primary)))
 				update_clipboard(primary,c->text,H_MODE_LIST);
-			if(is_clipboard_empty(clipboard))
+			else g_free (x);
+			if(NULL == (x=is_clipboard_empty(clipboard)))
 				update_clipboard(clipboard,c->text,H_MODE_LIST);
+			else g_free(x);
 		}
 	}
 	
@@ -1906,12 +2112,22 @@ int main(int argc, char *argv[])
   
   /* Initiate GTK+ */
   gtk_init(&argc, &argv);
-  
+   /* Read preferences */
+  read_preferences();
+#ifdef	DEBUG_UPDATE
+	if(get_pref_int32("debug_update")) debug_update=1;
+#endif
   /* Parse options */
 	opts=parse_options(argc, argv);
   if(NULL == opts)
    	return 1;
-	if(proc_find(PARCELLITE_PROG_NAME,PROC_MODE_EXACT,NULL)<2)	/**1 for me, and 1 for a running instance  */
+	if(opts->exit)
+		return 0;
+	mode=PROC_MODE_EXACT;
+	if(get_pref_int32("multi_user"))
+	  mode|=PROC_MODE_USER_QUALIFY;
+	/*g_printf("mode=0x%X\n",mode); */
+	if(proc_find(PARCELLITE_PROG_NAME,mode,NULL)<2)	/**1 for me, and 1 for a running instance  */
 		mode=PROG_MODE_DAEMON; /**first instance  */
 	else
 		mode=PROG_MODE_CLIENT; /**already running, just access fifos & exit.  */
